/******************************************************************************\
|*                                                                            *|
|*                           FingerCell Matcher 2.1                           *|
|*                                                                            *|
|* FncMatcherParams.h                                                         *|
|* FncMatcher parameters definition                                           *|
|*                                                                            *|
|* Copyright (C) 2006-2007 Neurotechnologija                                  *|
|*                                                                            *|
\******************************************************************************/

#ifndef FNC_MATCHER_PARAMS_H_INCLUDED
#define FNC_MATCHER_PARAMS_H_INCLUDED

#include <NParameters.h>

#ifdef N_CPP
extern "C"
{
#endif

// Main parameters

/**
 * \brief Static, Readonly, String | Returns name of the library
 * 
 * Identifier specifying library name static read-only parameter of type #N_TYPE_STRING.
 */
#define FNCMP_NAME         10 

/**
 * \brief Static, Readonly, UInt   | Returns high part of the library version
 * 
 *  Identifier specifying high part of library version static read-only parameter of type #N_TYPE_UINT. 
 *  Two high-order bytes of parameter value specify major version and two low-order bytes - minor version. 
 */
#define FNCMP_VERSION_HIGH 11 

/**
 * \brief Static, Readonly, UInt   | Returns low part of the library version
 * 
 * Identifier specifying low part of library version static read-only parameter of type #N_TYPE_UINT. 
 * Two high-order bytes of parameter value specify major (build) version and two low-order bytes - minor (release) version. 
 */
#define FNCMP_VERSION_LOW  12 

/**
 * \brief Static, Readonly, String | Returns copyright of the library
 * 
 * Identifier specifying library copyright static read-only parameter of type #N_TYPE_STRING.
 */
#define FNCMP_COPYRIGHT    13 

/**
 * \brief matching threshold (biggest allowed FAR)
 * 
 *  Identifier specifying matching threshold (biggest allowed FAR) parameter of type #N_TYPE_INT. 
 *  Parameter value is equal to -12 * log10(FAR) and must be not less than zero (for example, 48 for FAR = 0.01%). 
 */
#define FNCMP_MATCHING_THRESHOLD 200 

/**
 * \brief modulus of maximal rotation allowed between two matched NFRecords
 * 
 *  Identifier specifying modulus of maximal rotation allowed between two matched NFRecords parameter of type #N_TYPE_BYTE. 
 *  Parameter value is specified in 180/128 degrees units and can not be greater than 128 (+-180 degrees). 
 */
#define FNCMP_MAXIMAL_ROTATION   201

/**
 * \brief Int, parameter set
 * 
 * Identifier specifying mode (parameter value set) parameter of type #N_TYPE_UINT. Parameter value can be one of the FNCM_MODE_XXX. 
 */
#define FNCMP_MODE 1000 
	#define FNCM_MODE_GENERAL                           0 ///<The general mode.
	#define FNCM_MODE_DIGITALPERSONA_UAREU            100 ///<The mode for Digital Persona U.are.U scanner.
	#define FNCM_MODE_BIOMETRIKA_FX2000               200 ///<The mode for Biometrika FX2000 scanner.
	#define FNCM_MODE_BIOMETRIKA_FX3000               201 ///<The mode for Biometrika FX3000 scanner.
	#define FNCM_MODE_KEYTRONIC_SECUREDESKTOP         300 ///<The mode for Keytronic Secure Desktop scanner.
	#define FNCM_MODE_IDENTIX_TOUCHVIEW               400 ///<The mode for Identix TouchView scanner.
	#define FNCM_MODE_IDENTIX_DFR2090                 401 ///<The mode for Identix DFR 2090 scanner.
	#define FNCM_MODE_PRECISEBIOMETRICS_100CS         500 ///<The mode for Precise Biometrics 100CS scanner.
	#define FNCM_MODE_UPEK_TOUCHCHIP                  600 ///<The mode for UPEK TouchChip TCRU1C/TCRU2C sensors.
	#define FNCM_MODE_IDENTICATORTECHNOLOGY_DF90      700 ///<The mode for Identicator Technology DF90 scanner.
	#define FNCM_MODE_AUTHENTEC_AFS2                  800 ///<The mode for AuthenTec AF-S2 sensor.
	#define FNCM_MODE_AUTHENTEC_AES4000               810 ///<The mode for AuthenTec AES4000 sensor.
	#define FNCM_MODE_AUTHENTEC_AES2501B              811 ///<The mode for AuthenTec AES2501B sensor.
	#define FNCM_MODE_ATMEL_FINGERCHIP                900 ///<The mode for Atmel FingerChip sensor.
	#define FNCM_MODE_BMF_BLP100                     1000 ///<The mode for BMF BLP100 scanner.
	#define FNCM_MODE_SECUGEN_HAMSTER                1100 ///<The mode for SecuGen Hamster III scanner.
	#define FNCM_MODE_ETHENTICA                      1200 ///<The mode for Ethentica scanner.
	#define FNCM_MODE_CROSSMATCH_VERIFIER300         1300 ///<The mode for CrossMatch Verifier 300 scanner.
	#define FNCM_MODE_NITGEN_FINGKEY_HAMSTER         1400 ///<The mode for NITGEN Fingkey Hamster scanner.
	#define FNCM_MODE_TESTECH_BIOI                   1500 ///<The mode for Testech Bio-i sensor.
	#define FNCM_MODE_DIGENT_IZZIX                   1600 ///<The mode for Digent Izzix FD1000 scanner.
	#define FNCM_MODE_STARTEK_FM200                  1700 ///<The mode for Startek FM200 sensor.
	#define FNCM_MODE_FUJITSU_MBF200                 1800 ///<The mode for Fujitsu MBF200 scanner.
	#define FNCM_MODE_FUTRONIC_FS80                  1900 ///<The mode for Futronic's FS80 scanner.
	#define FNCM_MODE_LIGHTUNING_LTTC500             2000 ///<The mode for LighTuning LTT-C500 scanner.
	#define FNCM_MODE_TACOMA_CMOS                    2100 ///<The mode for Tacoma CMOS sensor.

#ifdef N_CPP
}
#endif

#endif // !FNC_MATCHER_PARAMS_H_INCLUDED
